# -*- coding: utf-8 -*-
"""Version information for this package."""
__version__ = "3.0.4"
VERSION = __version__
""":obj:`str`: Version of package."""

__url__ = "https://github.com/Axonius/axonius_api_client"
URL = __url__
""":obj:`str`: URL of package."""

__author__ = "Axonius"
AUTHOR = __author__
""":obj:`str`: Auth of package."""

__title__ = "axonius_api_client"
TITLE = __title__
""":obj:`str`: Title of package."""

__project__ = "axonius_api_client"
PROJECT = __project__
""":obj:`str`: Name of package."""

__author_email__ = "support@axonius.com"
AUTHOR_EMAIL = __author_email__
""":obj:`str`: Author email of package."""

__description__ = "Axonius API client for Python"
DESCRIPTION = __description__
""":obj:`str`: Description of package."""

__docs__ = "https://axonius-api-client.readthedocs.io/en/latest/"
DOCS = __docs__
""":obj:`str`: Link to the documentation for this package."""

__license__ = "MIT"
LICENSE = __license__
""":obj:`str`: License of package."""

__copyright__ = "Copyright Axonius 2020"
COPYRIGHT = __copyright__
""":obj:`str`: Copyright of package."""
