# -*- coding: utf-8 -*-
"""Test suite."""
import pytest

from axonius_api_client.api.parsers.fields import get_type_norm
from axonius_api_client.exceptions import ApiError


def test_get_type_norm_error():
    """Pass."""
    field = {"type": "badwolf", "name": "badwolf"}
    with pytest.raises(ApiError):
        get_type_norm(field=field)
